@@ -1,9 +1,9 @@
 ##' Extract variable names from latent variable model
-##' 
+##'
 ##' Extract exogenous variables (predictors), endogenous variables (outcomes),
 ##' latent variables (random effects), manifest (observed) variables from a
 ##' \code{lvm} object.
-##' 
+##'
 ##' \code{vars} returns all variables of the \code{lvm}-object including
 ##' manifest and latent variables. Similarily \code{manifest} and \code{latent}
 ##' returns the observered resp. latent variables of the model.
@@ -18,30 +18,30 @@
 ##' \code{latent} returns a vector with the names of the latent variables in
 ##' \code{x}. The assigment function \code{latent<-} can be used to change the
 ##' latent status of variables in the model.
-##' 
+##'
 ##' @aliases vars vars.lvm vars.lvmfit latent latent<- latent.lvm latent<-.lvm
 ##' latent.lvmfit latent.multigroup manifest manifest.lvm manifest.lvmfit
 ##' manifest.multigroup exogenous exogenous<- exogenous.lvm exogenous<-.lvm
 ##' exogenous.lvmfit exogenous.multigroup endogenous endogenous.lvm
 ##' endogenous.lvmfit endogenous.multigroup
 ##' @usage
-##' 
+##'
 ##' vars(x,...)
-##' 
+##'
 ##' endogenous(x,...)
-##' 
+##'
 ##' exogenous(x,...)
-##' 
+##'
 ##' manifest(x,...)
-##' 
+##'
 ##' latent(x,...)
-##' 
+##'
 ##' \method{exogenous}{lvm}(x,silent = FALSE, xfree = TRUE,...) <- value
-##' 
+##'
 ##' \method{exogenous}{lvm}(x,latent=FALSE,index=TRUE,...)
-##' 
+##'
 ##' \method{latent}{lvm}(x,clear=FALSE,...) <- value
-##' 
+##'
 ##' @param x \code{lvm}-object
 ##' @param latent Logical defining whether latent variables without parents
 ##' should be included in the result
@@ -58,7 +58,7 @@
 ##' \code{\link{latent}}, \code{\link{exogenous}}, \code{\link{vars}}
 ##' @keywords models regression
 ##' @examples
-##' 
+##'
 ##' g <- lvm(eta1 ~ x1+x2)
 ##' regression(g) <- c(y1,y2,y3) ~ eta1
 ##' latent(g) <- ~eta1
@@ -91,7 +91,7 @@ function(x,...) UseMethod("vars")
 ##' @export
 vars.list <- function(x,...) {
   varlist <- c()
-  for (i in 1:length(x)) {
+  for (i in seq_along(x)) {
     varlist <- c(varlist, vars(x[[i]]))
   }
   varlist <- unique(varlist)
